package net.wanji.utc.controller;

/**
 * @author Kent HAN
 * @date 2022/12/1 15:07
 */

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.RequiredArgsConstructor;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dao.entity.CrossLightsPO;
import net.wanji.databus.dto.CrossIdDateTimeDTO;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.databus.vo.LightsStatusVO2;
import net.wanji.utc.cache.CrossPhaseDirTurnCache;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.po.hk.CrossPhaseDirTurnPojo;
import net.wanji.utc.service.runninginfo.SignalStatusService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.ws.rs.core.MediaType;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 运行状态、告警、灯态信息接口
 *
 * @date 2022/11/15 9:38
 */
@Api(value = "运行状态、告警、灯态信息接口", description = "运行状态、告警、灯态信息接口")
@RequestMapping("/signalStatus")
@RestController
@RequiredArgsConstructor
public class SignalStatusController {
    private final SignalStatusService signalStatusService;

    @AspectLog(description = "运行状态、告警数据", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/runningStatusAlarm", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "运行状态、告警数据", notes = "运行状态、告警数据", response = SignalStatusLogPO.class,
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SignalStatusLogPO.class)
    })
    public JsonViewObject runningStatusAlarm() throws Exception {
        List<SignalStatusLogPO> signalStatusLogPOList = signalStatusService.runningStatusAlarm();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(signalStatusLogPOList);

    }

    @AspectLog(description = "灯态数据", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/lightStatus", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "灯态数据", notes = "灯态数据", response = LightsStatusVO.class,
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = LightsStatusVO.class)
    })
    public JsonViewObject lightStatus() throws Exception {
        List<LightsStatusVO> lightsStatusVOList = signalStatusService.lightStatus();
        return JsonViewObject.newInstance().success(lightsStatusVOList);
    }

    @AspectLog(description = "灯态数据V2", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/lightStatusV2", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "灯态数据V2", notes = "灯态数据V2", response = LightsStatusVO.class,
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = LightsStatusVO.class)
    })
    public JsonViewObject lightStatusV2() throws Exception {
        List<LightsStatusVO2> collect = null;
        try {
            //List<LightsStatusVO> lightsStatusVOList = signalStatusService.lightStatus4StaticScheme();
            List<LightsStatusVO> lightsStatusVOList = signalStatusService.lightStatus();
            List<LightsStatusVO2> listResult = new ArrayList<>(lightsStatusVOList.size());

            for (LightsStatusVO lightsStatusVO : lightsStatusVOList) {
                if (Objects.isNull(lightsStatusVO)) {
                    continue;
                }
                Integer cyclePhaseCountDown = lightsStatusVO.getCyclePhaseCountDown();
                LightsStatusVO2 lightsStatusVO2 = new LightsStatusVO2();
                BeanUtils.copyProperties(lightsStatusVO, lightsStatusVO2);
                Map<String, Object> dirLampGroupMap = lightsStatusVO.getDirLampGroupMap();
                List<LightsStatusVO2.DirInfo> dirList = new ArrayList<>();
                Map<String, Object> phaseMap = lightsStatusVO.getPhaseMap();
                for (Map.Entry<String, Object> groupEntry : dirLampGroupMap.entrySet()) {
                    LightsStatusVO2.DirInfo dirInfo = new LightsStatusVO2.DirInfo();
                    String dir = groupEntry.getKey();
                    Map<String, String> turnColorMap = (Map<String, String>) groupEntry.getValue();
                    List<LightsStatusVO2.TurnInfo> turnList = new ArrayList<>();
                    for (Map.Entry<String, String> turnEntry : turnColorMap.entrySet()) {
                        String turn = turnEntry.getKey();
                        String color = turnEntry.getValue();
                        LightsStatusVO2.TurnInfo turnInfo = new LightsStatusVO2.TurnInfo();
                        turnInfo.setTurn(String.valueOf(turn));
                        turnInfo.setColor(color);
                        String key = dir.concat(Constants.SEPARATOR_UNDER_LINE).concat(turn);
                        Object countDown = phaseMap.get(key);
                        if (Objects.isNull(countDown)) {
                            countDown = cyclePhaseCountDown;
                        }
                        if (Objects.isNull(cyclePhaseCountDown)) {
                            countDown = lightsStatusVO.getCycleCountDown();
                        }
                        turnInfo.setCountDown((int) countDown);
                        turnList.add(turnInfo);
                    }
                    dirInfo.setDir(dir);
                    dirInfo.setTurnList(turnList);
                    dirList.add(dirInfo);
                }
                lightsStatusVO2.setDirLampGroupMapList(dirList);

                /**
                 * 处理灯态，获取当前相位号灯态
                 */
                //if (StringUtils.equals("13N0C0B5P30", lightsStatusVO.getCrossId())) {
                    List<LightsStatusVO2.DirInfo> curPhaseDirList = getCurPhaseDirList(lightsStatusVO2);
                    lightsStatusVO2.setCurPhaseLampGroupMapList(curPhaseDirList);
                //}

                listResult.add(lightsStatusVO2);
            }
            collect = listResult.stream().filter(vo -> StringUtils.equals(vo.getCrossId(), "13N0C0B5P30")).collect(Collectors.toList());
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return JsonViewObject.newInstance().success(collect);
    }


    /**
     * 通过当前灯态获取当前相位的灯态方向转向倒计时信息
     * @param lightsStatusVO2
     * @return
     */
    private static List<LightsStatusVO2.DirInfo> getCurPhaseDirList(LightsStatusVO2 lightsStatusVO2) {
        List<LightsStatusVO2.DirInfo> curPhaseList = new ArrayList<>();
        try {
            String crossId = lightsStatusVO2.getCrossId();
            String schemeId = lightsStatusVO2.getSchemeId();
            String phaseId = lightsStatusVO2.getPhaseId();
            List<LightsStatusVO2.DirInfo> dirLampGroupMapList = lightsStatusVO2.getDirLampGroupMapList();
            Map<String, List<CrossPhaseDirTurnPojo>> crossInfoMap = CrossPhaseDirTurnCache.crossInfoMap;
            if (!crossInfoMap.isEmpty()) {
                List<CrossPhaseDirTurnPojo> crossPhaseDirTurnPojos = crossInfoMap.get(crossId);
                if (!CollectionUtils.isEmpty(crossPhaseDirTurnPojos)) {
                    for (CrossPhaseDirTurnPojo crossPhaseDirTurnPojo : crossPhaseDirTurnPojos) {
                        String curCrossId = crossPhaseDirTurnPojo.getCrossId();
                        Integer curSchemeNo = crossPhaseDirTurnPojo.getSchemeNo();
                        Integer curPhaseNo = crossPhaseDirTurnPojo.getPhaseNo();
                        if (StringUtils.equals(crossId, curCrossId) && Objects.equals(Integer.valueOf(schemeId), curSchemeNo)
                                && Objects.equals(Integer.valueOf(phaseId), curPhaseNo)) {
                            Map<Integer, List<CrossLightsPO>> dirTurnMap = crossPhaseDirTurnPojo.getDirTurnMap();
                            if (!dirTurnMap.isEmpty()) {
                                for (Map.Entry<Integer, List<CrossLightsPO>> item : dirTurnMap.entrySet()) {
                                    LightsStatusVO2.DirInfo dirInfo = new LightsStatusVO2.DirInfo();
                                    Integer dir = item.getKey();
                                    List<LightsStatusVO2.TurnInfo> turnList = new ArrayList<>();
                                    List<CrossLightsPO> value = item.getValue();
                                    if (!CollectionUtils.isEmpty(value)) {
                                        for (CrossLightsPO crossLightsPO : value) {
                                            Integer type = crossLightsPO.getType();
                                            Integer curTurn = crossLightsPO.getTurn();

                                            String color = "";
                                            for (LightsStatusVO2.DirInfo info : dirLampGroupMapList) {
                                                String statusDir = info.getDir();
                                                if (StringUtils.equals(statusDir, String.valueOf(dir))) {
                                                    List<LightsStatusVO2.TurnInfo> statusTurnList = info.getTurnList();
                                                    for (LightsStatusVO2.TurnInfo turnInfo : statusTurnList) {
                                                        String statusTurn = turnInfo.getTurn();
                                                        if (StringUtils.equals(String.valueOf(curTurn), statusTurn)) {
                                                            color = turnInfo.getColor();
                                                        }
                                                    }
                                                }
                                            }

                                            String turn = "";
                                            // 过滤行人
                                            if (Objects.equals(3, type)) {
                                                continue;
                                            }
                                            if (Objects.equals(2, type)) {
                                                for (int i = 1; i <= 2 ; i++) {
                                                    // 设置转向信息
                                                    LightsStatusVO2.TurnInfo turnInfo = new LightsStatusVO2.TurnInfo();
                                                    turnInfo.setTurn(String.valueOf(i));
                                                    turnInfo.setColor(color);
                                                    turnInfo.setCountDown(0);
                                                    turnList.add(turnInfo);
                                                }
                                            } else if (Objects.equals(1, type)){
                                                turn = String.valueOf(curTurn);
                                                LightsStatusVO2.TurnInfo turnInfo = new LightsStatusVO2.TurnInfo();
                                                turnInfo.setTurn(turn);
                                                turnInfo.setColor(color);
                                                turnInfo.setCountDown(0);
                                                turnList.add(turnInfo);
                                            }

                                        }
                                    }
                                    if (!CollectionUtils.isEmpty(turnList)) {
                                        dirInfo.setDir(String.valueOf(dir));
                                        dirInfo.setTurnList(turnList);
                                        curPhaseList.add(dirInfo);
                                    }
                                }
                            }
                        }
                    }
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
            //log.error("获取当前相位灯态倒计时失败：", e);
        }

        return curPhaseList;
    }


    @AspectLog(description = "灯态数据历史数据列表", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/lightStatusHist", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "灯态数据历史数据列表", notes = "灯态数据历史数据列表", response = LightsStatusVO2.class,
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = LightsStatusVO2.class)
    })
    public JsonViewObject lightStatusHist(String crossId, Integer startBatchTime, Integer endBatchTime) throws Exception {
        List<LightsStatusVO2> lightsStatusVOList = signalStatusService.lightStatusHist(crossId, startBatchTime, endBatchTime);
        return JsonViewObject.newInstance().success(lightsStatusVOList);
    }

    @AspectLog(description = "查询某一时间灯态历史数据", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/lightStatusOneHist", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "查询某一时间灯态历史数据", notes = "查询某一时间灯态历史数据", response = LightsStatusVO2.class,
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = LightsStatusVO2.class)
    })
    public JsonViewObject lightStatusOneHist(@RequestBody @Valid CrossIdDateTimeDTO crossIdDateTimeDTO) throws Exception {
        return signalStatusService.lightStatusOneHist(crossIdDateTimeDTO);
    }
}
