package net.wanji.utc.task;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dto.CrossSchedulesDTO;
import net.wanji.databus.dto.PlanSectionDTO;
import net.wanji.databus.dto.SchemePhaseLightsDTO;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.service.staticinfo.StaticInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;

import java.util.ArrayList;
import java.util.List;

/**
 * 定时同步方案等静态信息
 *
 * @author zhouleilei
 * @date 2024/12/25 13:01
 */
//@Component
@Slf4j
@SuppressWarnings("all")
public class SchemeUpdateTask {
    @Autowired
    StaticInfoService staticInfoService;

    @Scheduled(cron = "0 30 0 * * ?")
    public void runningStatusAlarm() throws Exception {
        for (CrossInfoPO crossInfoPO : CrossInfoCache.crossInfoList) {
            if (ObjectUtil.isEmpty(crossInfoPO.getCode())) {
                continue;
            }
            Integer manufacturerId = crossInfoPO.getManufacturerId();
            String id = crossInfoPO.getId();
            //同步 方案数据-方案信息、相位信息、灯组信息
            SchemePhaseLightsDTO schemePhaseLightsDTO = new SchemePhaseLightsDTO();
            schemePhaseLightsDTO.setCrossId(id);
            staticInfoService.schemePhaseLights(schemePhaseLightsDTO);

            //同步 计划数据-计划信息、时段信息
            PlanSectionDTO planSectionDTO = new PlanSectionDTO();
            planSectionDTO.setCrossId(id);
            planSectionDTO.setPlanNo(-1);
            staticInfoService.planSection(planSectionDTO);

            //同步 时间表数据
            CrossSchedulesDTO crossSchedulesDTO = new CrossSchedulesDTO();
            if (manufacturerId == 22) {
                //海信
                crossSchedulesDTO.setManufacturerCode("HISENSE");
            } else if (manufacturerId == 23) {
                crossSchedulesDTO.setManufacturerCode("EHUALU");
            }
            List<String> list = new ArrayList<>();
            list.add(id);
            crossSchedulesDTO.setCrossIdList(list);
            staticInfoService.crossSchedules(crossSchedulesDTO);
            log.info("路口 {} 定时同步完成", id);
        }
    }

}
