package net.wanji.utc.util;

import cn.hutool.core.util.HexUtil;
import freemarker.template.utility.StringUtil;
import net.wanji.common.utils.tool.ScheduleUtil;
import net.wanji.utc.common.typeenum.HisenseTurnEnum;
import net.wanji.utc.common.typeenum.HiseseDirEnum;

import java.util.ArrayList;
import java.util.List;

/**
 * @author duanruiming
 * @date 2023/06/20 15:07
 */
public class CommonUtils {

    /**
     * 将数组转化为16进制字符串
     *
     * @param array
     * @return
     */
    public static String getHex4Array(int[] array) {
        StringBuilder sb = new StringBuilder();
        for (int i : array) {
            sb.append(i);
        }
        Integer binary = Integer.valueOf(sb.reverse().toString(), 2);
        return HexUtil.toHex(binary);
    }

    /**
     * 将时间转化为16进制 00：00 --> 061E0
     *
     * @param time
     * @return
     */
    public static String getHourMinuteHex(String time) {
        StringBuffer sb = new StringBuffer();
        String[] split = time.split(":");
        String hour = String.format("%02x", Integer.valueOf(split[0]));
        String minute = String.format("%02x", Integer.valueOf(split[1]));
        return sb.append(hour).append(minute).toString();
    }

    /**
     * 将相位转化为64位数组，在转化为16位的16进制
     *
     * @param param
     * @return
     */
    public static String getBinary(Integer param) {
        int[] array = new int[64];
        array[param - 1] = 1;
        StringBuffer resultSb = new StringBuffer();
        for (int i = 0; i < array.length / 8; i++) {
            StringBuffer sb = new StringBuffer();
            for (int j = 0; j < 8; j++) {
                sb.append(array[i * 8 + j]);
            }
            String hex = String.format("%02x", Integer.parseInt(sb.reverse().toString(), 2));
            resultSb.insert(0, hex);
        }
        return StringUtil.leftPad(resultSb.toString(), 16, "0");
    }

    /**
     * 将单个参数转化为4位二进制
     *
     * @param param
     * @return
     */
    public static Integer get4BinaryInteger(Integer param) {
        int[] array = new int[4];
        int index = param % 4;
        if (param % 4 != 0) {
            array[4 - index] = 1;
        } else {
            array[0] = 1;
        }
        StringBuffer sb = new StringBuffer();
        for (int j = 0; j < 4; j++) {
            sb.append(array[j]);
        }
        return Integer.parseInt(sb.toString(), 2);
    }

    /**
     * 通过灯组编号获取灯组转向， 1 左转 2 直 3 右 20 行人
     *
     * @param lightId
     * @return
     */
    public static Integer getTurnByLightsId(Integer lightId) {
        Integer turn = null;
        if (lightId % 4 == 0) {
            turn = 1;
        }
        if (lightId % 4 == 1) {
            turn = 2;
        }
        if (lightId % 4 == 2) {
            turn = 3;
        }
        if (lightId % 4 == 3) {
            turn = 20;
        }
        return turn;
    }

    public static Integer getDirByLightsId(Integer lightsId) {
        Integer dir = null;
        if (lightsId < 4) {
            dir = 1;
        }
        if (lightsId >= 4 && lightsId < 8) {
            dir = 3;
        }
        if (lightsId >= 8 && lightsId < 12) {
            dir = 5;
        }
        if (lightsId >= 12 && lightsId < 16) {
            dir = 7;
        }
        return dir;
    }

    /**
    * @Description 将字符串分成八个一组，不够的最前面补0
    * @Param [input]
    * @return java.util.List<java.lang.String>
    **/

    public static List<String> groupStringFromRight(String input) {
        List<String> list = new ArrayList<>();
        if (input == null || input.isEmpty()) {
            return null;
        }
        // 计算需要多少个完整的8字符组
        int fullGroups = input.length() / 8;
        // 计算最后一个组中的字符数
        int lastGroupSize = input.length() % 8;

        StringBuilder result = new StringBuilder(input);

        String str = "";
        if (lastGroupSize > 0) {
            // 计算需要补多少个0
            int padding = 8 - lastGroupSize;
            for (int i = 0; i < padding; i++) {
                str = str+"0";
            }
        }
        input = str+result.toString();
        int size =input.length()/8;

        for (int i = 0; i < size; i++) {
            String substring = input.substring(i * 8, (i + 1) * 8);
            list.add(substring);
        }

        return list;
    }


    public static void main(String[] args) {
        int l = 6442253;
        String binaryString = Integer.toBinaryString(l);
        List<String> readUtf8Lines = groupStringFromRight(binaryString);
        System.out.println(readUtf8Lines.size());
        for (String readUtf8Line : readUtf8Lines) {
            System.out.println(readUtf8Line);
            String dir = readUtf8Line.substring(0, 3); // 高三位 方向
            String turn = readUtf8Line.substring(3); // 低五位 转向
            System.out.println(ScheduleUtil.binaryIntToDecimalism(dir)+"_"+ScheduleUtil.binaryIntToDecimalism(turn));
            Integer wjLightsDir = HiseseDirEnum.getWjLightsDir(Integer.parseInt(dir, 2));
            Integer wjLightsTurn = HisenseTurnEnum.getWjLightsTurn(Integer.parseInt(turn, 2));
            System.out.println("wjLightsDir "+wjLightsDir);
            System.out.println("wjLightsTurn "+wjLightsTurn);
            System.out.println("======================================");

        }
    }
}
